;==========================================================
; GROUP.LSP Copyright 1994 by Looking Glass Microproducts
;==========================================================
; Collect selected entities into an anonymous block 
;==========================================================
(defun C:GROUP (/ ERROR OLD-ERROR ITEM MAPSET NOTRANS 
                  PUSHVARS POPVARS SYSVARS BITSET 
                  READY_LAYER)
   ;=======================================================
   ; Error Handler
   ;
   (defun ERROR (S)
      (if (not
             (member
                S
                '("Function cancelled" "console break")
             )
          )
         (princ S)
      )
      (command)
      (command ".undo" "end")
      (command ".undo" "group")
      (POPVARS)
      (princ)
   )
   ;=======================================================
   ; Set and Save System Variables
   ;
   (defun PUSHVARS (VLIST)
      (foreach PAIR VLIST
         (setq
            SYSVARS (cons
                       (cons
                          (strcase (car PAIR))
                          (getvar
                             (car PAIR)
                          )
                       )
                       SYSVARS
                    )
         )
         (if (cdr PAIR) (setvar (car PAIR) (cdr PAIR)))
      )
   )
   ;=======================================================
   ; Restore System Variables
   ;
   (defun POPVARS ()
      (foreach PAIR SYSVARS (setvar (car PAIR) (cdr PAIR)))
      (setq *error* OLD-ERROR)
      (setq SYSVARS nil)
   )
   ;=======================================================
   ; Disallow transparent invocation of routine.
   ;
   (defun NOTRANS ()
      (cond
         ((member (getvar "cmdactive") '(nil 0)))
         ((prompt
          "\nThis command may not be invoked transparently."
          )
         )
      )
   )
   ;=======================================================
   ; Item from association list
   ;
   (defun ITEM (A B) (cdr (assoc A B)))
   ;=======================================================
   ; Return T if any bit of A matches B
   ; 
   (defun BITSET (A B) (/= 0 (logand A B)))
   ;=======================================================       
   ; Ready Layer for Use
   ;
   (defun READY_LAYER (LNAME / LENTRY)
      (if (setq LENTRY (tblsearch "layer" LNAME))
         (progn
            (if (BITSET (ITEM 70 LENTRY) 1)
               (command ".layer" "thaw" LNAME "")
            )
            (if (BITSET (ITEM 70 LENTRY) 4)
               (command ".layer" "unlock" LNAME "")
            )
            (if (minusp (ITEM 62 LENTRY))
               (command ".layer" "on" LNAME "")
            )
         )
         (command ".layer" "new" LNAME "")
      )
   )
   ;=======================================================
   ; Main routine
   ;
   (defun GROUP (/ SS N BLOCKNAME CLAYER)
      (cond
         ((null (setq SS (ssget))))
         (t
            (setq CLAYER (getvar "clayer"))
            (READY_LAYER "0")
            (if (/= "0" CLAYER)
               (command ".layer" "set" "0" "")
            )
            (setq N 0)
            (while (progn
                      (setq
                        N         (1+ N)
                        BLOCKNAME (strcat "GROUP_" (itoa N))
                      )
                      (tblsearch "block" BLOCKNAME)
                   )
            )
            (setvar "highlight" 0)
            (command
               ".block" BLOCKNAME "none" '(0 0 0) SS ""
            )
            (command
               ".insert" BLOCKNAME "none" '(0 0 0) 1 1 0
            )
            (if (/= "0" CLAYER)
               (command ".layer" "set" CLAYER "")
            )
            (setq SS nil)
         )
      )
   )
   ;=======================================================
   ; Body of c:group  
   ;=======================================================
   (if (NOTRANS)
      (progn
         (setq OLD-ERROR *error* *error* ERROR)
         (PUSHVARS '(("cmdecho" . 0) ("highlight")))
         (command ".undo" "group")
         (GROUP)
         (command ".undo" "end")
         (POPVARS)
      )
   )
   (princ)
)
(princ
   (strcat
      "  GROUP.LSP v1.0 (Copyright 1994 by "
      "Looking Glass Microproducts) loaded."
   )
)
(princ)
